/*
*-+*-+*-+*-+*-+*-+*-+*-+*-+*-+
	Vandarth!  (Altures Primus)
    Game Initialization Support Routines

    This functions support Game Creation and Initialization
*/


void *ThreadInfoLine(void *data) {

	print("       -[SysInfo]  Started...\n");
	ThreadCountIncrement();
	
	while (!GCF_Terminate) {
      pthread_mutex_lock(&M_Screen);
   	wclear(InfoLine);
	   mvwprintw(InfoLine,0,3,"Mem Free: %d   Swap Free: %d   Hash: [%d:%d]",MemFree(),MemSwapFree(),HT_UnitIDoLen,HT_UnitIDeLen);
      pthread_mutex_unlock(&M_Screen);
   	refreshall();
		sleep(2);
	}
	
	print("       -[SysInfo]  Shutdown...\n");
	ThreadCountDecrement();
	return;
};

void NCursesInit(void) {
	initscr();
	keypad(stdscr,TRUE);
	cbreak();
	noecho();
	nonl();
	scrollok(stdscr,TRUE);
	leaveok(stdscr,TRUE);	
	if (has_colors()) {
		start_color();
		init_pair(COLOR_BLACK, COLOR_BLACK, COLOR_BLACK);
		init_pair(COLOR_GREEN, COLOR_GREEN, COLOR_BLACK);
		init_pair(COLOR_RED, COLOR_RED, COLOR_BLACK);
		init_pair(COLOR_CYAN, COLOR_CYAN, COLOR_BLACK);
		init_pair(COLOR_WHITE, COLOR_WHITE, COLOR_BLACK);
		init_pair(COLOR_MAGENTA, COLOR_MAGENTA, COLOR_BLACK);
		init_pair(COLOR_BLUE, COLOR_BLUE, COLOR_BLACK);
		init_pair(COLOR_YELLOW, COLOR_YELLOW, COLOR_BLACK);
		A_Norm = A_NORMAL|COLOR_PAIR(COLOR_GREEN);
		A_HOk = A_REVERSE|A_BOLD|COLOR_PAIR(COLOR_GREEN);
		A_HErr = A_REVERSE|A_BOLD|A_BLINK|COLOR_PAIR(COLOR_RED);
		A_SysMesg = A_DIM|COLOR_PAIR(COLOR_BLACK);
		A_Prompt = A_BOLD|COLOR_PAIR(COLOR_CYAN);
	}
	if (LINES<24 || COLS<80) {
		endwin();
		printf("\nERROR: Server must be run at least a 80x24 terminal!\n");
		exit(1);
	}	
	// Window Initialization
	Mesg = newwin(LINES-5,COLS,3,0);
	Status = newwin(1,COLS,LINES-1,0);
	InfoLine = newwin(1,COLS,1,0);
	CommandLine = newwin(1,COLS,LINES-2,0);
	scrollok(Mesg,TRUE);
	wsetscrreg(Mesg, 0, LINES-6);
	idlok(Mesg, TRUE);
	keypad(Mesg, TRUE);
	wbkgdset(CommandLine,A_DIM|COLOR_PAIR(COLOR_YELLOW));
	wbkgdset(InfoLine,A_DIM|COLOR_PAIR(COLOR_BLACK));
	wbkgdset(Status,A_BOLD|A_REVERSE|COLOR_PAIR(COLOR_GREEN));
	wclear(Status);wclear(Mesg);wclear(InfoLine);wclear(CommandLine);
};




void GameInit(char *StringBuffer) {
   int x;

	attrset(A_BOLD|COLOR_PAIR(COLOR_YELLOW)|A_REVERSE);
	mvprintw(0,0,"%s",Padl("  VanDarthe!",StringBuffer,COLS));
	sprintf(StringBuffer," Game Server ver. %d.%d.%d \n",GAME_MAJOR_VERSION,GAME_MINOR_VERSION,GAME_BUILD_DATE);
	attrset(A_BOLD|COLOR_PAIR(COLOR_CYAN)|A_REVERSE);
	mvprintw(0,COLS-strlen(StringBuffer),"%s",StringBuffer);
	D_Status("Initializing NCURSES...");
	if (DELAYS)	sleep(1);
	D_Status("Initializing Game Server...");
	if (DELAYS)	sleep(1);
	B(1);H(1);print("Welcome!");B(0);H(0);
	printr("(Please Wait while initializing Game...)");N();N();
	D_Status("Initializing Queues...    ");
		// Command Queue Initialization
		B(1);print("Queues");B(0);print(":\n");
		B(1);print("    [Command Queue]   ");B(0);
		if (pthread_mutex_lock(&MutEx_CQ)) {endwin();perror("CQ Master Lock");exit(1);};
		print(" [Lock] ");
		if ((CommandQueue = malloc((sizeof(struct CQ))*CQUEUE_INIT_LENGTH))==NULL) {perror("CQ Init Alloc");exit(1);};
		CQ_Length = CQUEUE_INIT_LENGTH;
		CQ_Head = 0;   // Queue Init Blank Conditions
		CQ_Tail = 0;
		if (DELAYS)	sleep(1);
		H(1);print(" [OK] ");H(0);
		if (pthread_mutex_unlock(&MutEx_CQ)) {endwin();perror("CQ Master Unlock");exit(1);};
		print(" [Unlock] ");
	N();		
		// Movement Queue Initialization
		B(1);print("    [Movement Queue]  ");B(0);
		if (pthread_mutex_lock(&MutEx_MQ)) {endwin();perror("MQ Master Lock");exit(1);};
		print(" [Lock] ");
		if ((MovementQueue = malloc(sizeof(struct MQ)*MQUEUE_INIT_LENGTH))==NULL) {perror("MQ Init Alloc");exit(1);};
		MQ_Length = MQUEUE_INIT_LENGTH;
		MQ_Head = 0;   // Queue Init Blank Conditions
		MQ_Tail = 0;
		if (DELAYS)	sleep(2);
		H(1);print(" [OK] ");H(0);
		if (pthread_mutex_unlock(&MutEx_MQ)) {endwin();perror("MQ Master Unlock");exit(1);};
		print(" [Unlock] ");
	N();
	
	
	B(1);print("\nMemory Requirements:\n");B(0);
	sprintf(StringBuffer,"   Global Server Data: %d\n   World Data: %d\n   Thread Minimum: %d\n",sizeof(struct GlobalServerData),sizeof(struct MapTile)*GW_HEIGHT*GW_WIDTH,PTHREAD_STACK_MIN);
	print(StringBuffer);
	
	D_Status("Setting up Threads...");
	if (pthread_attr_init(&attr)) {endwin();printf("01w");exit(1);};
	if (pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED)) {endwin();printf("01x");exit(1);};
	if (pthread_attr_setstacksize(&attr, (PTHREAD_STACK_MIN))) {endwin();printf("01x1");exit(1);};
	if (pthread_create(&T_InfoLine,&attr,ThreadInfoLine,(void*) &x)) {endwin();printf("01e");exit(1);};
	if (DELAYS) sleep(2);

	// Setup UnitID Hash Table
	HT_UnitID_Setup(20);
};

int SetUnits(char *String) {
	static unsigned char CurArrPos = 0;
	#define UNITPARAMLENGTH 	23

	char *Tokens[UNITPARAMLENGTH];
	char x;
	char Sea;
	
	if (Tokenize("\t",Tokens,String)!=UNITPARAMLENGTH) {
		return(1);
	};	
	GameData.UnitData[CurArrPos].Type = (char) atoi(Tokens[0]);
	Sea = ((192&((char) atoi(Tokens[0])))==64) ? 1:0;
	strcpy(GameData.UnitData[CurArrPos].Name,Tokens[1]);
	GameData.UnitData[CurArrPos].Life = (short int) atoi(Tokens[2]);
	GameData.UnitData[CurArrPos].Heal = (char) atoi(Tokens[3]);
	GameData.UnitData[CurArrPos].CostProduction = (short int) atoi(Tokens[4]);
	GameData.UnitData[CurArrPos].CostMoney = (short int) atoi(Tokens[5]);
	GameData.UnitData[CurArrPos].UpkeepCost = (unsigned int) atoi(Tokens[6]);
	GameData.UnitData[CurArrPos].Movement = (unsigned char) ((float) (atof(Tokens[11]) *10));
	GameData.UnitData[CurArrPos].SightRange = (unsigned char) atoi(Tokens[12]);
	GameData.UnitData[CurArrPos].AttackSequence = (unsigned char) atoi(Tokens[22]);
	GameData.UnitData[CurArrPos].SightRange = (unsigned char) atoi(Tokens[12]);
	if (GSD_MaxSight<GameData.UnitData[CurArrPos].SightRange)
		GSD_MaxSight=GameData.UnitData[CurArrPos].SightRange;
	
	
	GameData.UnitData[CurArrPos].DefenseProjSmall = (unsigned char) ((float) (atof(Tokens[18]) *100));
	GameData.UnitData[CurArrPos].DefenseProjLarge = (unsigned char) ((float) (atof(Tokens[19]) *100));
	GameData.UnitData[CurArrPos].DefenseExplosive = (unsigned char) ((float) (atof(Tokens[20]) *100));
	GameData.UnitData[CurArrPos].DefenseIncedentiary = (unsigned char) ((float) (atof(Tokens[21]) *100));
	
	GameData.UnitData[CurArrPos].AttackAir = (unsigned char) atoi(Tokens[17]);
	GameData.UnitData[CurArrPos].AttackProjSmall = (unsigned char) atoi(Tokens[13]);
	GameData.UnitData[CurArrPos].AttackProjLarge = (unsigned char) atoi(Tokens[14]);
	if (Sea) {
		GameData.UnitData[CurArrPos].Transport = (unsigned char) atoi(Tokens[15]);
		GameData.UnitData[CurArrPos].AttackSea = (unsigned char) atoi(Tokens[16]);
	} else {
		GameData.UnitData[CurArrPos].Transport = 0;
		GameData.UnitData[CurArrPos].AttackSea = 0;
		GameData.UnitData[CurArrPos].AttackExplosive = (unsigned char) atoi(Tokens[15]);
		GameData.UnitData[CurArrPos].AttackIncedentiary = (unsigned char) atoi(Tokens[16]);
	}
	
	if ((*(Tokens[7]))=='-') { // Not a stealth unit
		GameData.UnitData[CurArrPos].StealthCostCloak=GameData.UnitData[CurArrPos].StealthCostDecloak=GameData.UnitData[CurArrPos].StealthAttack=GameData.UnitData[CurArrPos].StealthUpkeep=0;
	} else {
		GameData.UnitData[CurArrPos].StealthCostCloak = (unsigned char) ((float) (atof(Tokens[7]) *10));
		GameData.UnitData[CurArrPos].StealthCostDecloak = (unsigned char) ((float) (atof(Tokens[9]) *10));
		GameData.UnitData[CurArrPos].StealthAttack = (unsigned char) (atoi(Tokens[8]));
		GameData.UnitData[CurArrPos].StealthUpkeep = (unsigned char) ((float) (atof(Tokens[10]) *10));
	}

//	GameData.UnitData[CurArrPos].StealthCostCloak = (unsigned char) (((unsigned int) atoi(Tokens[7]))*10);
	
//	print("%d|%s|%d|%d|%d|%d",Sea,GameData.UnitData[CurArrPos].Name,GameData.UnitData[CurArrPos].AttackProjSmall,GameData.UnitData[CurArrPos].Transport,GameData.UnitData[CurArrPos].AttackSea,GameData.UnitData[CurArrPos].AttackAir);
//	getchar();
	for(x=0;x<UNITPARAMLENGTH;x++) free(Tokens[x]);
	CurArrPos++;
	
//	for(x=0;x<UNITPARAMLENGTH;x++) printf("\n%s",Tokens[x]);
   return(0);
};



int SetTraps(char *String) {
	static unsigned char CurArrPos = 0;
	#define TRAPPARAMLENGTH 	8
	char x;
	char *Tokens[TRAPPARAMLENGTH];
	
	if (Tokenize("\t",Tokens,String)!=TRAPPARAMLENGTH) {
		return(1);
	};	
	strcpy(GameData.TrapData[CurArrPos].Name,Tokens[0]);
	GameData.TrapData[CurArrPos].Upkeep = (unsigned char) (((unsigned int) atoi(Tokens[1]))/10);
	GameData.TrapData[CurArrPos].ConstHours = (unsigned char) atoi(Tokens[2]);
	GameData.TrapData[CurArrPos].ConstMoney = (unsigned char) (((unsigned int) atoi(Tokens[3]))/100);
	GameData.TrapData[CurArrPos].ActivationRange = (unsigned char) atoi(Tokens[4]);
	GameData.TrapData[CurArrPos].DamageUnits = (unsigned char) (((unsigned int) atoi(Tokens[5]))/10);
	GameData.TrapData[CurArrPos].DamageVehicles = (unsigned char) (((unsigned int) atoi(Tokens[6]))/10);
	GameData.TrapData[CurArrPos].DamageRange = (unsigned char) atoi(Tokens[7]);
		
	for(x=0;x<TRAPPARAMLENGTH;x++) free(Tokens[x]);
	CurArrPos++;
	
   return(0);
};

int SetSettlements(char *String) {
	static unsigned char CurArrPos = 0;
	#define SETTLEMENTPARAMLENGTH 	3
	char x;
	char *Tokens[SETTLEMENTPARAMLENGTH];
	if (Tokenize("\t",Tokens,String)!=SETTLEMENTPARAMLENGTH) {
		return(1);
	};	
	strcpy(GameData.SettlementData[CurArrPos].Name,Tokens[0]);
	GameData.SettlementData[CurArrPos].Upkeep = (unsigned char) (((unsigned int) atoi(Tokens[1]))/100);
	GameData.SettlementData[CurArrPos].Evolution = (unsigned char) (((unsigned int) atoi(Tokens[2]))/100);
	for(x=0;x<SETTLEMENTPARAMLENGTH;x++) free(Tokens[x]);
	CurArrPos++;

   return(0);
};

int SetRadars(char *String) {
	static unsigned char CurArrPos = 0;
	#define RADARPARAMLENGTH 	3
	char x;
	char *Tokens[RADARPARAMLENGTH];
	if (Tokenize("\t",Tokens,String)!=RADARPARAMLENGTH) {
		return(1);
	};	
	GameData.RadarData[CurArrPos].Upkeep = (unsigned char) (((unsigned int) atoi(Tokens[0]))/100);
	GameData.RadarData[CurArrPos].StealthRange = (unsigned char) atoi(Tokens[1]);
	GameData.RadarData[CurArrPos].SightRange = (unsigned char) atoi(Tokens[2]);
	for(x=0;x<RADARPARAMLENGTH;x++) free(Tokens[x]);
	CurArrPos++;

   return(0);
};

int SetBases(char *String) {
	static unsigned char CurArrPos = 0;
	#define BASEPARAMLENGTH 	3
	char x;
	char *Tokens[BASEPARAMLENGTH];
	if (Tokenize("\t",Tokens,String)!=BASEPARAMLENGTH) {
		return(1);
	};	
	GameData.BaseData[CurArrPos].Upkeep = (unsigned char) (((unsigned int) atoi(Tokens[0]))/100);
	GameData.BaseData[CurArrPos].Evolution = (unsigned char) (((unsigned int) atoi(Tokens[1]))/10);
	GameData.BaseData[CurArrPos].Manufacture = (unsigned char) atoi(Tokens[2]);
	for(x=0;x<BASEPARAMLENGTH;x++) free(Tokens[x]);
	CurArrPos++;

   return(0);
};

int SetTerrains(char *String) {
	static unsigned char CurArrPos = 0;
	#define TERRAINPARAMLENGTH 	7
	char x;
	char *Tokens[TERRAINPARAMLENGTH];
	if (Tokenize("\t",Tokens,String)!=TERRAINPARAMLENGTH) {
		return(1);
	};	
	strcpy(GameData.TerrainData[CurArrPos].Name,Tokens[0]);
	GameData.TerrainData[CurArrPos].Attack = (unsigned char) ((unsigned int) ((float) (atof(Tokens[1])*10)));
	GameData.TerrainData[CurArrPos].DefenseUnit = (unsigned char) ((unsigned int) ((float) (atof(Tokens[2])*10)));
	GameData.TerrainData[CurArrPos].DefenseVehicle = (unsigned char) ((unsigned int) ((float) (atof(Tokens[3])*10)));
	GameData.TerrainData[CurArrPos].MovementUnit = (unsigned char) ((unsigned int) ((float) (atof(Tokens[4])*10)));
	GameData.TerrainData[CurArrPos].MovementVehicle = (unsigned char) ((unsigned int) ((float) (atof(Tokens[5])*10)));
	GameData.TerrainData[CurArrPos].Stealth = (unsigned char) atoi(Tokens[6]);
	for(x=0;x<TERRAINPARAMLENGTH;x++) free(Tokens[x]);
	CurArrPos++;

   return(0);
};

int SetBarriers(char *String) {
	static unsigned char CurArrPos = 0;
	#define BARRIERPARAMLENGTH 	2
	char x;
	char *Tokens[BARRIERPARAMLENGTH];
	if (Tokenize("\t",Tokens,String)!=BARRIERPARAMLENGTH) {
		return(1);
	};	
	GameData.BarrierData[CurArrPos].Upkeep = (unsigned char) (((unsigned int) atoi(Tokens[0]))/100);
	GameData.BarrierData[CurArrPos].Range = (unsigned char) atoi(Tokens[1]);
	for(x=0;x<BARRIERPARAMLENGTH;x++) free(Tokens[x]);
	CurArrPos++;

   return(0);
};


char ParseSettings(FILE *Buffer)  {
	char *String;
	char x, *Temp;
	char Sequence=0;
   char Flag=0;
   int DataLineNo=0;
   char *IdSequence[7];


	D_Status("Parsing Settings File...");
   IdSequence[0] = strdup("[UNITS]");
   IdSequence[1] = strdup("[TRAPS]");
   IdSequence[2] = strdup("[SETTLEMENTS]");
   IdSequence[3] = strdup("[RADAR]");
   IdSequence[4] = strdup("[BASES]");
   IdSequence[5] = strdup("[TERRAIN]");
   IdSequence[6] = strdup("[BARRIER]");
//  	IdSequence[7] = strdup("[AIRSTRIP]");

	Temp = malloc(sizeof(char)*100);
	String = malloc(sizeof(char)*1000);
	while (!feof(Buffer)) {
		ReadLine(Buffer, String, 0);
//				printx("[");printx(String);print("]\n");getch();
		if (String!=NULL) {
			DataLineNo++;
			strtrim(String);
			if (*String!='\0') {
				if (*String!='[' && Flag==0) {
					sprintf(Temp,"    **Error in Data Line Number:%d: Invalid Statement.\n",DataLineNo);
					print(Temp);Err();
					free(Temp);
					return(1);
				} else
					if (*(String)!='[' && Flag>0) {   //  DATA UNDER Section
						switch (Flag) {
							case 1:
								if ((x=SetUnits(String))) {
									sprintf(Temp,"    **Error:%d,%d: Invalid Statement.\n",DataLineNo,x);
									print(Temp);Err();
									free(Temp);
									return(1);
								};
								break;
							case 2:
								if ((x=SetTraps(String))) {
									sprintf(Temp,"    **Error:%d,%d: Invalid Statement.\n",DataLineNo,x);
									print(Temp);Err();
									free(Temp);
									return(1);
								};
								break;
							case 3:
								if ((x=SetSettlements(String))) {
									sprintf(Temp,"    **Error:%d,%d: Invalid Statement.\n",DataLineNo,x);
									print(Temp);Err();
									free(Temp);
									return(1);
								};
								break;
							case 4:
								if ((x=SetRadars(String))) {
									sprintf(Temp,"    **Error:%d,%d: Invalid Statement.\n",DataLineNo,x);
									print(Temp);Err();
									free(Temp);
									return(1);
								};								
								break;
							case 5:
								if ((x=SetBases(String))) {
									sprintf(Temp,"    **Error:%d,%d: Invalid Statement.\n",DataLineNo,x);
									print(Temp);Err();
									free(Temp);
									return(1);
								};								
								break;
							case 6:
								if ((x=SetTerrains(String))) {
									sprintf(Temp,"    **Error:%d,%d: Invalid Statement.\n",DataLineNo,x);
									print(Temp);Err();
									free(Temp);
									return(1);
								};
								break;
							case 7:
								if ((x=SetBarriers(String))) {
									sprintf(Temp,"    **Error:%d,%d: Invalid Statement.\n",DataLineNo,x);
									print(Temp);Err();
									free(Temp);
									return(1);
								};								
								break;
							default:
								sprintf(Temp,"    **Error in Data Line Number:%d: Extra Section Identifier.\n",DataLineNo);
								print(Temp);Err();
								free(Temp);
								return(1);
						}; 	
					} else
						if (*(String)=='[' && Flag>=0) {		// New Section
							if (strcmp(String,IdSequence[Flag])) {   // Right Flag?
									sprintf(Temp,"    **Error in Data Line Number:%d: Error in Section Sequence or Unexpected Identifier.\n",DataLineNo);
									print(Temp);Err();
									free(Temp);
									return(1);
							};							
							Flag++;
						} else {	//  Error in File Structure
							sprintf(Temp,"    **Error in Data Line Number:%d: Unexpected Statement Sequence.\n",DataLineNo);
							print(Temp);Err();
							free(Temp);
							return(1);
						};
					// }
			};
		};
	};
	free(String);
	free(Temp);
	print("    **Settings File in perfect condition.");Ok();N();N();
};

char ParseMap(FILE *InputFile) {
	int x, y, pass;
	int CurChar;	
	long FileStartTag;
	char TerrainTable[TERRAINCOLORS], *Temp;
	
	D_Status("Parsing the Map File...");
	TerrainTable[TERRAINPLAIN]=TPLAIN;
	TerrainTable[TERRAINMOUNTAIN]=TMOUNTAIN;
	TerrainTable[TERRAINHILL]=THILL;
	TerrainTable[TERRAINJUNGLE]=TJUNGLE;
	TerrainTable[TERRAINSWAMP]=TSWAMP;
	TerrainTable[TERRAINGRASSLAND]=TGRASSLAND;
	TerrainTable[TERRAINSEA]=TSEA;
	Temp = malloc(sizeof(char)*100);
	CurChar=0;
	while (!(feof(InputFile) || CurChar==TERRAINMARKER)) {
		CurChar=fgetc(InputFile);
	}
	if (feof(InputFile)) {
		print("    **Input File Not a Map or not in correct format.\n");Err();
		return(1);
	}
  	if (CurChar==TERRAINMARKER) {
		FileStartTag = ftell(InputFile);
		sprintf(Temp,"    **System Start Tag Located at file position %ld.\n",FileStartTag);
		print(Temp);
	}
	if (pthread_mutex_lock(&MutEx_GW)) {endwin();perror("    **GW Master Lock");exit(1);};
	pass=1;
	for(y=0;y<GW_HEIGHT;y++) {
		for (x=0;x<GW_WIDTH;x++)
			if (pass==1) {
				pass=2;
				GameWorld[x][y].Terrain = TSEA;
				GameWorld[x][y].IsFortressed = 0;
				GameWorld[x][y].IsTrapped = 0;
				GameWorld[x][y].InSight = 0;
				GameWorld[x][y].InBarrier  = 0;
				GameWorld[x][y].HasUnits  = 0;
				GameWorld[x][y].HasBuilding = 0;
				GameWorld[x][y].HasBase = 0;
				GameWorld[x][y].Data=NULL;
//				printf(" ");
			} else {
				CurChar=fgetc(InputFile);
//				if ((x>=XMinRange && x<=XMaxRange) && (y>=YMinRange && y<=YMaxRange))
//					printf("%c",((char) CurChar)+65);
				GameWorld[x][y].Terrain = TerrainTable[CurChar];
				GameWorld[x][y].IsFortressed = 0;
				GameWorld[x][y].IsTrapped = 0;
				GameWorld[x][y].InSight = 0;
				GameWorld[x][y].InBarrier  = 0;
				GameWorld[x][y].HasUnits  = 0;
				GameWorld[x][y].HasBuilding = 0;
				GameWorld[x][y].HasBase = 0;
				GameWorld[x][y].Data=NULL;
//				fprintf(OutputFile,"%c%c|%c\n",(char) x,(char) y,(char) CurChar);
			}
//		if (y<100)
//			printf("\n");		
	}
	if (pthread_mutex_unlock(&MutEx_GW)) {endwin();perror("    **GW Master Unlock");exit(1);};
	
	CurChar=fgetc(InputFile);
	if (feof(InputFile)) {
		print("    **Input File in perfect condition.");Ok();N();N();
		free(Temp);
		return(0);
	} else {
		sprintf(Temp,"    **Input File has excess data.  Incorrect Map Type... (%ld)\n\n",ftell(InputFile));
		print(Temp);Err();N();N();
		free(Temp);
		return(1);
	}	
};


void NewGameInit(char *StringBuffer, FILE *FileBuffer) {
	X1;print("\n\n\n-=New Game=-\n");Y1;
	D_Status("Enter the filename of the Game Settings for the new game.");
	wattron(Mesg,A_Prompt);		
	printx("   Enter Game Settings File");wattroff(Mesg,A_Prompt);print(" [ServerDefaultData.gam]: ");
	StringBuffer=AskString(StringBuffer,"ServerDefaultData.gam",100);		
	while (((FileBuffer=fopen(StringBuffer,"r"))==NULL)) {
		printx("    **File Open Error: ");printx(strerror(errno));print("\n");N();
		wattron(Mesg,A_Prompt);		
		printx("   Enter Game Settings File");wattroff(Mesg,A_Prompt);print(" [ServerDefaultData.gam]: ");
		StringBuffer=AskString(StringBuffer,"ServerDefaultData.gam",100);
				
	}
	print("    **Game Settings File Opened.  Parsing...\n");
	if (DELAYS) sleep(1);
	ParseSettings(FileBuffer);
	fclose(FileBuffer);
	
	D_Status("Enter the filename of the Map Definition File for the new game.");
	wattron(Mesg,A_Prompt);print("   Enter Game World Map File");wattroff(Mesg,A_Prompt);print(" [SPMap.bmp]: ");
	StringBuffer=AskString(StringBuffer,"SPMap.bmp",100);
	while (((FileBuffer=fopen(StringBuffer,"r"))==NULL)) {
		printx("    **File Open Error: ");printx(strerror(errno));print("\n");N();
		wattron(Mesg,A_Prompt);
		wattron(Mesg,A_Prompt);print("   Enter Game World Map File");wattroff(Mesg,A_Prompt);print(" [SPMap.bmp]: ");
		StringBuffer=AskString(StringBuffer,"SPMap.bmp",100);
	}
	print("    **Game World Map File Opened.  Parsing...\n");
	if (DELAYS) sleep(2);
	ParseMap(FileBuffer);
	fclose(FileBuffer);
	
	// Game Variable Initialization.
	GameData.ActivePlayers = NULL;
	GameData.InactivePlayers = NULL;
	GameData.NoActivePlayers = 0;
	GameData.NoInactivePlayers = 0;
	GameData.SettlementSetupTime = SETUP_SETTLEMENT;
	GameData.BarrierSetupTime = SETUP_BARRIER;
	GameData.RadarSetupTime = SETUP_RADAR;
	GameData.AirStripSetupTime = SETUP_AIRSTRIP;
};